"""
Copyright (C) 2018 Cumulus Networks, inc.

All Rights reserved.

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/

Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

from nclu import PENDING_COMMANDS_FILE
import logging
import os

log = logging.getLogger("netd")


def is_net_del_all():
    """
    Return a boolean indicating if the command "net del all" is the last pending command.

    Note that the is an imperfect way of determining if the device will be in the default,
    "net del all" state after commiting the pending changes.  Consider for example:

        net del all
        net add interface swp2
        net del interface swp2

    This is equivalent to "net del all", but this function will return False.
    """

    assert os.path.isfile(PENDING_COMMANDS_FILE), "Don't call this function if there are no pending changes."
    saw_net_del_all = False

    # Get the last pending command.
    with open(PENDING_COMMANDS_FILE) as f:
        for line in f:
            if line.rstrip().endswith("net del all"):
                saw_net_del_all = True

    # "line" now holds the value of the last line in pending_commands.txt.
    # It looks like "cumulus\t2018-09-12 15:36:33.772083\tnet del all\n".
    net_del_all = line.rstrip().endswith("net del all")

    if not net_del_all and saw_net_del_all:
        log.warning("\"net del all\" is pending, but it is not the most recent command.  The configuration might not "
                    "match the installation state after \"net commit\" for some services.")

    if net_del_all:
        log.debug("Executing a \"net del all\".")

    return net_del_all
